#pragma once

// Include all necessary headers from the ADTF SDK
#include <adtffiltersdk/adtf_filtersdk.h>

// For simplicity use the necessary namespaces
using namespace adtf::util;
using namespace adtf::ucom;
using namespace adtf::base;
using namespace adtf::streaming;
using namespace adtf::filter;

// To implement a Filter we subclass adtf::filter::cFilter
class cPluginDescriptionFilter: public cFilter
{
public:
    
    // If use this macro, the Class ID and Label Name will be exported:
    //      <name>plugin_description_filter.filter.adtf_guides.cid</name>
    //      <label>Plugin Description Filter</label>
    ADTF_CLASS_ID_NAME(cPluginDescriptionFilter,
                       "plugin_description_filter.filter.adtf_guides.cid",
                       "Plugin Description Filter");

    // If use this macro, the Plugin Description Generator will add required interfaces:
    //      <required_interfaces>
    //          <interface_description>
    //              <iid>reference_clock.giant.streaming.adtf.iid< / iid>
    //          </interface_description>
    //      </required_interfaces>
    ADTF_CLASS_DEPENDENCIES(REQUIRE_INTERFACE(adtf::services::IReferenceClock));

public:
    // In the constructor, the Plugin Description Generator will look for some basic plugin information, see implementation.
    cPluginDescriptionFilter();

    // If use this function, dynamic input pins will be enabled in Plugin Description
    // node:
    //      <dynamic_input_pins>true</dynamic_input_pins>
    tResult RequestDynamicInputPin(const char* /*strName*/,
                                   const adtf::ucom::ant::iobject_ptr<const adtf::streaming::ant::IStreamType>& /*pType*/) override 
    {
        RETURN_NOERROR;
    };

    // If use this function, dynamic output pins will be enabled in Plugin Description
    // node:
    //      <dynamic_output_pins>true</dynamic_input_pins>
    tResult RequestDynamicOutputPin(const char* /*strName*/,
                                    const adtf::ucom::ant::iobject_ptr<const adtf::streaming::ant::IStreamType>& /*pType*/) override
    {
        RETURN_NOERROR;
    };

    // If use this function, dynamic input binding pins will be enabled in Plugin Description
    // node:
    //      <dynamic_input_binding_pins>true</dynamic_input_binding_pins>
    tResult RequestDynamicInterfaceClient(const char* /*strName*/,
                                          const adtf::ucom::ant::iobject_ptr<const adtf::streaming::ant::IBindingType>& /*pType*/) override
    {
        RETURN_NOERROR;
    };

    // If use this function, dynamic output binding pins will be enabled in Plugin Description
    // node:
    //      <dynamic_output_binding_pins>true</dynamic_output_binding_pins>
    tResult RequestDynamicInterfaceServer(const char* /*strName*/,
                                          const adtf::ucom::ant::iobject_ptr<const adtf::streaming::ant::IBindingType>& /*pType*/) override
    {
        RETURN_NOERROR;
    };

private:
    // Here are all the Pin Reader/Writer for our Filter
    ISampleReader* m_pReader = nullptr;
    ISampleWriter* m_pWriter = nullptr;

    // Here are all the Property Variables for our Filter
    property_variable<double> m_fFloatProperty = 3.1415;
    property_variable<cString> m_strProperty = { "i am a string" };

};
